/******************************************************************************
* intersection_test.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
/*
 * intersection_test.h
 *
 *  Created on: Jul 1, 2010
 *      Author: pajor
 */

#ifndef INTERSECTION_TEST_H_
#define INTERSECTION_TEST_H_

#include "../dataStructures/graph.h"
#include "../dataStructures/path.h"

#include <boost/graph/graph_traits.hpp>


namespace schematization {

// This tests whether a graph is a simple graph.
template<typename graphType>
bool is_simple_graph(const graphType &graph) {
	if (num_vertices(graph) < 3)
		return true;

	typedef typename graph_traits<graphType>::out_edge_iterator out_edge_iterator;
	typedef typename graphType::vertex_descriptor vertex;

	out_edge_iterator current_e, inner_e, last_e, inner_last_e;
	for (tie(current_e, last_e) = edges(graph); current_e != last_e; ++current_e) {
		for (tie(inner_e, inner_last_e) = edges(graph); inner_e != inner_last_e; ++inner_e) {
			if (current_e == inner_e)
				continue;
			// Get the lines
			line l1 = get_edge_line(current_e, graph);
			line l2 = get_edge_line(inner_e, graph);

			if (tools::is_intersecting(l1, l2))
				return false;
		}
	}

	return true;
}

}

#endif /* INTERSECTION_TEST_H_ */
