/******************************************************************************
* boundingBox.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
/*
 * boundingBox.h
 *
 *  Created on: May 4, 2010
 *      Author: pajor
 */

#ifndef BOUNDINGBOX_H_
#define BOUNDINGBOX_H_

#include "point.h"

namespace schematization {

class boundingBox {
public:

	double top, left, bottom, right;

	boundingBox() : top(0), left(0), bottom(0), right(0) {}
	boundingBox(const point p) :
		top(p.y),
		left(p.x),
		bottom(p.y),
		right(p.x) {}
	boundingBox(const point _topleft, const point _bottomright) :
		top(_topleft.y),
		left(_topleft.x),
		bottom(_bottomright.y),
		right(_bottomright.x) {}

	void extend(const point p) {
		top    = std::min(top, p.y);
		left   = std::min(left, p.x);
		bottom = std::max(bottom, p.y);
		right  = std::max(right, p.x);
	}
	void extend(const boundingBox &b) {
		top    = std::min(top, b.top);
		left   = std::min(left, b.left);
		bottom = std::max(bottom, b.bottom);
		right  = std::max(right, b.right);
	}
	bool contains(const point p) const {
		if (p.x < left || p.x > right)
			return false;
		if (p.y < top || p.y > bottom)
			return false;
		return true;
	}
	double get_extent_x() const {
		return std::abs(right - left);
	}
	double get_extent_y() const {
		return std::abs(bottom - top);
	}

	
};




}

#endif /* BOUNDINGBOX_H_ */
