/******************************************************************************
* mercatorProjection.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
/*
 * mercatorProjection.h
 *
 *  Created on: 09.06.2010
 *      Author: pajor
 */

#ifndef MERCATORPROJECTION_H_
#define MERCATORPROJECTION_H_

#include "../types.h"
#include "point.h"
#include "latLng.h"

#include "tools.h"

namespace schematization {

class mercatorProjection {
public:

	/*
	 * Convert from points to lat lng
	 */
	static inline latLng point_to_latlng(const point &p)  {
		latLng latlng(0,0);

		latlng.lng = (p.x - pixel_origin.x) / pixels_per_lon_deg;
		double lat_rad = (p.y - pixel_origin.y) / (-pixels_per_lon_rad);
		latlng.lat = tools::rad_to_deg(2.0 * std::atan(std::exp(lat_rad)) - pi / 2.0);
		return latlng;
	}

	/*
	 * Convert from lat lng to point. This code is ported from the google maps API
	 */
	static inline point latlng_to_point(const latLng &latlng) {
		point p(0, 0);
		p.x = pixel_origin.x + latlng.lng * pixels_per_lon_deg;
		double siny = tools::bound(std::sin(tools::deg_to_rad(latlng.lat)), -0.9999, 0.9999);
		p.y = pixel_origin.y + 0.5 * std::log((1.0 + siny) / (1.0 - siny)) * (-pixels_per_lon_rad);
		return p;
	}


protected:

	// This is the "world tile" size
	static const double MERCATOR_SIZE;
	static const point pixel_origin;
	static const double pixels_per_lon_rad;
	static const double pixels_per_lon_deg;
};

// Define the static constants
const double mercatorProjection::MERCATOR_SIZE = 256.0;
const point mercatorProjection::pixel_origin(mercatorProjection::MERCATOR_SIZE / 2.0, mercatorProjection::MERCATOR_SIZE / 2.0);
const double mercatorProjection::pixels_per_lon_rad = mercatorProjection::MERCATOR_SIZE / (2.0 * pi);
const double mercatorProjection::pixels_per_lon_deg = mercatorProjection::MERCATOR_SIZE / 360.0;

}

#endif /* MERCATORPROJECTION_H_ */
