/******************************************************************************
* rectangle.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
#ifndef RECTANGLE_H
#define RECTANGLE_H

#include "tools.h"

namespace schematization {

class rectangle {

public:

	rectangle(): top(0), left(0), bottom(0), right(0) {}

	rectangle(const point _topleft, const point _bottomright) :
	top(_topleft.y),
	left(_topleft.x),
	bottom(_bottomright.y),
	right(_bottomright.x) {}
		
	void move_by(double delta_x, double delta_y) 
	{
		top += delta_x;
		bottom += delta_x;
		left += delta_y;
		right += delta_y;
	}

	bool intersection_with(const rectangle& b) {
		//one rectangle inside another?
		vector<point> points;
		points.push_back(point(left, bottom));
		points.push_back(point(left, top));
		points.push_back(point(right, top));
		points.push_back(point(right, bottom));
		
		vector<point> b_points;
		b_points.push_back(point(b.left, b.bottom));
		b_points.push_back(point(b.left, b.top));
		b_points.push_back(point(b.right, b.top));
		b_points.push_back(point(b.right, b.bottom));

		for (size_t i = 0; i < 4; i++)
		{
			if (b.left <= points[i].x && points[i].x <= b.right)
				if (b.top <= points[i].y && points[i].y <= b.bottom)
					return true;
			if (left <= b_points[i].x && b_points[i].x <= right)
				if (top <= b_points[i].y && b_points[i].y <= bottom)
					return true;
		}


		//intersection of edges of the rectangles
		for (size_t i = 0; i < 4; i++) {
			schematization::line l_1(points[i], points[ (i + 1) % 4]);
			for (size_t j = 0; j < 4; j++) {
				schematization::line l_2(points[i], points[ (i + 1) % 4]);

				if (tools::is_intersecting(l_1, l_2))
					return true;
			}

		}

		return false;
	}
	
	double top, bottom, left, right;

};

}
#endif