/******************************************************************************
* types.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
/*
 * types.h
 *
 *  Created on: May 18, 2010
 *      Author: pajor
 */

#ifndef TYPES_H_
#define TYPES_H_

#include "geometry/color.h"

#include <vector>
#include <string>
#include <boost/math/constants/constants.hpp>

namespace schematization {

// Orthogonal order types
enum orderType {
	X_ORDER,
	Y_ORDER
};

// Type representing angles for schematization. Must be fraction of 90
typedef double angleType;

// Type representing street names
typedef std::vector<std::string> streetNamesType;


const size_t category_context = 10;
const double category_to_thickness[] = {10, 10, 10, 10, 10, 10, 10, 10, 10, 14, 5};
const color category_to_color[] = {col_lightred, col_orange, col_yellow, col_green, col_lightgrey, col_lightgrey, col_lightgrey, col_lightgrey, col_lightgrey, col_lightblue, col_ultralightgrey};
const double category_to_eps[] = {1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0};
//const double category_to_eps[] = {1.0, 1.0/4.0, 1.0/8.0, 1.0/16.0, 1.0/32.0, 1.0/32.0, 1.0/64.0, 1.0/64.0, 1.0/64.0, 1/64.0};
const int category_to_min_length[] = {90, 80, 70, 60, 50, 40, 35, 35, 35, 35, 35, 35, 35}; //dots


// Default paper sizes
const double paper_resolution = 72.0 * 0.0393700787; // dots per mm
const double paper_size_w = 210 * paper_resolution; // in dots
const double paper_size_h = 287 * paper_resolution; // in dots
const double paper_margin_h = 10; // in dots
const double path_margin    = 15; // in dots

// Default extents
const double base_line_width = 1.0; // in dots
const double base_border_extent = 1.0; // in dots
const double base_node_thickness = 1.0; // in dots
const double base_node_thickness_separator = 2.25; // in dots
const double base_street_name_coefficient = 2.0/3.0; // no unit
const double base_glyph_width = base_street_name_coefficient * 1.5; // 1.5 dots?
const double base_footline_font_size = 8; // in dots

// Minimum extents
const double min_edge_length = 25.0; // in dots
const double min_label_margin = 15; // in dots
const double min_edge_dist   = 15.0; // in dots

const double pi = boost::math::constants::pi<double>();

}

#endif /* TYPES_H_ */
